
/*
 * This is serial library API include file.
 * History:
 * Date		Author			Comment
 * 06-02-2006	Victor Yu.		Create it.
 */
#include	<asm/termios.h>

/*	BAUD rate setting	*/
#ifndef B50
#define B50		0x00
#endif
#ifndef B75
#define B75		0x01
#endif
#ifndef B110
#define B110		0x02
#endif
#ifndef B134
#define B134		0x03
#endif
#ifndef B150
#define B150		0x04
#endif
#ifndef B300
#define B300		0x05
#endif
#ifndef B600
#define B600		0x06
#endif
#ifndef B1200
#define B1200		0x07
#endif
#ifndef B1800
#define B1800		0x08
#endif
#ifndef B2400
#define B2400		0x09
#endif
#ifndef B4800
#define B4800		0x0A
#endif
#ifndef B7200
#define B7200		0x0B
#endif
#ifndef B9600
#define B9600		0x0C
#endif
#ifndef B19200
#define B19200		0x0D
#endif
#ifndef B38400
#define B38400		0x0E
#endif
#ifndef B57600
#define B57600		0x0F
#endif
#ifndef B115200
#define B115200 	0x10
#endif
#ifndef B230400
#define B230400 	0x11
#endif
#ifndef B460800
#define B460800 	0x12
#endif
#ifndef B921600
#define B921600 	0x13
#endif

/*	MODE setting		*/
#define BIT_5		0x00			/* Word length define	*/
#define BIT_6		0x01
#define BIT_7		0x02
#define BIT_8		0x03
#define BIT_MASK	0x03

#define STOP_1		0x00			/* Stop bits define	*/
#define STOP_2		0x04

#define P_EVEN		0x18			/* Parity define	*/
#define P_ODD		0x08
#define P_SPC		0x38
#define P_MRK		0x28
#define P_NONE		0x00
#define P_MASK		0x38

/*	MODEM CONTROL setting	*/
#define C_DTR		0x01
#define C_RTS		0x02

/*	MODEM LINE STATUS	*/
#define S_CTS		0x01
#define S_DSR		0x02
#define S_RI		0x04
#define S_CD		0x08

/* 	flow control 		*/
#define CTS_FLOWCTRL	0x01
#define RTS_FLOWCTRL	0x02
#define TX_XONXOFF	0x04
#define RX_XONXOFF	0x08
#define HW_FLOWCTRL	(CTS_FLOWCTRL|RTS_FLOWCTRL)
#define SW_FLOWCTRL	(TX_XONXOFF|RX_XONXOFF)
#define FLOWCTRL_MASK	(HW_FLOWCTRL|SW_FLOWCTRL)
#define NONE_FLOWCTRL	0

/*	flush data buffer 	*/
#define FLUSH_INPUT	0
#define FLUSH_OUTPUT	1
#define FLUSH_INOUT	2

/*	serial device interface		*/
#ifndef RS232_MODE
#define RS232_MODE		0
#endif
#ifndef RS485_2WIRE_MODE
#define RS485_2WIRE_MODE	1
#endif
#ifndef RS422_MODE
#define RS422_MODE		2
#endif
#ifndef RS485_4WIRE_MODE
#define RS485_4WIRE_MODE	3
#endif

/* error code */
#define SIO_OK		0
#define SIO_BADPORT	-1	/* no such port or port not opened */
#define SIO_NODATA	-4	/* no data to read or no buffer to write */
#define SIO_OPENFAIL	-5	/* no such port or port has be opened */
#define SIO_RTS_BY_HW	-6      /* RTS can't set because H/W flowctrl */
#define SIO_BADPARM	-7	/* bad parameter */
#define SIO_FAIL	-10	/* function run result fail */
#define SIO_NOINIT	-11	/* not initialize the API */

#ifdef __cplusplus
extern "C" {
#endif

//
// Basic function phototype
//

/*
 * int sio_init(void) 
 *
 * To serial device API initialize. Just do one time when the system enable.
 * Input:
 * 	None
 * Outout:
 * 	Follow the error code.
 */
int  sio_init(void);

/*
 * void sio_release(void)
 *
 * To release the all serial device. Just do one time when the system shotdown.
 * Input:
 * 	None
 * Output:
 * 	None
 */
void sio_release(void);

/*
 * int sio_open(int port)
 *
 * To open the serial device. You must first call this function when you want to use the serial device.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * Output:
 * 	Follow the error code.
 */
int  sio_open(int port);

/*
 * int sio_close(int port)
 *
 * To close the serial device. You must last call this function when you don't want to use the serial device.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * Output:
 * 	Follow the error code.
 */
int  sio_close(int port);

/*
 * int sio_read(int port, char *buf, int len)
 *
 * To read/receive the data from serial device. 
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * 	char *buf	- save the received data buffer.
 * 	int len		- want to receive max data length
 * Output:
 * 	>=0		- has received data length
 * 	< 0		- Follow the error code.
 */
int  sio_read(int port, char *buf, int len);

/*
 * int sio_write(int port, char *buf, int len)
 *
 * To write/transmit the data to seiral device.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * 	char *buf	- data buffer which wants to transmit.
 * 	int len		- want to transmit max data length
 * Output:
 * 	>=0		- has transmited data length
 * 	< 0		- Follow the error code.
 */
int  sio_write(int port, char *buf, int len);
#define sio_putb(x, y, z)	sio_write(x, y, z)

/*
 * int sio_getch(int port)
 *
 * To read one byte data from serial device.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * Output:
 * 	>=0 		- the data, just one byte
 * 	< 0		- Follow the error code.
 */
int  sio_getch(int port);

/*
 * int sio_putch(int port, int term)
 *
 * To transmit one byte data to serial device.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * 	int term	- want to transmit data, just one byte
 * Output:
 * 	Follow the error code.
 */
int  sio_putch(int port, int term);

/*
 * int sio_read(int port, char *buf, int len)
 *
 * To read/receive the data from serial device until the term byte or length. 
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * 	char *buf	- save the received data buffer.
 * 	int len		- want to receive max data length
 * 	int term	- the termial byte.
 * Output:
 * 	>=0		- has received data length
 * 	< 0		- Follow the error code.
 */
int  sio_linput(int port, char *buf, int len, int term);
/*
 * int sio_ioctl(int port, int baud, int mode)
 *
 * To set the serial device baud rate (speed), data bits, parity bit & stop bit.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * 	int baud	- want to set baud rate or speed.
 * 	int mode	- data bits, parity bit & stop bit. The define is above.
 * Output:
 * 	Follow the error code.
 */
int  sio_ioctl(int port, int baud, int mode);

/*
 * int sio_flush(int port, int func)
 *
 * To flush (clear) input or output data buffer on the serial device driver. The FIFO which is on the UART can not flush.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * 	int func	- want to flush input or output. The define is above.
 * Output:
 * 	Follow the error code.
 */
int  sio_flush(int port, int func);

/*
 * long sio_iqueue(int port)
 *
 * To get the data bytes which have received in the serial device driver buffer.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * Output:
 * 	>= 0		- how many bytes in the serial device driver buffer.
 * 	< 0		- follow the rerror code
 */
long sio_iqueue(int port);

/*
 * long sio_oqueue(int port)
 *
 * To get the data bytes which have not transmited in the serial device driver buffer.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * Output:
 * 	>= 0		- how many bytes in the serial device driver buffer.
 * 	< 0		- follow the rerror code
 */
long sio_oqueue(int port);

/*
 * int sio_lstatus(int port)
 *
 * To get the modem status at now.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * Output:
 * 	>= 0		- the modem status at now. The define is above.
 * 	< 0		- follow the rerror code
 */
int  sio_lstatus(int port);

/*
 * int sio_lctrl(int port, int mode)
 *
 * To control the modem status RTS/DTR on or off. 
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * 	int mode	- assing DTR or RTS. If you assign it, it will be on. If you don't assign it, it will be off.
 * 			  You can not control the RTS signal when the hardware flow control is enabled.
 * Output:
 * 	Follow the error code.
 */
int  sio_lctrl(int port, int mode);

/*
 * int sio_break_ex(int port, int time)
 *
 * To send break signal.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * 	int time	- watn to send break time, the unit is msecond
 * Output:
 * 	Follow the error code.
 */
int  sio_break_ex(int port, int time);

/*
 * int sio_flowctrl(int port, int mode)
 *
 * To set the serial device flow control. You can enable hardware or software flow control to gurantee the data will not be lost.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * 	int mode	- the flow control mode. The define is above.
 * Output:
 * 	Follow the error code.
 */
int  sio_flowctrl(int port, int mode);

/*
 * long sio_getbaud(int port)
 *
 * To get the set baud rate (speed) at now.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * Output:
 * 	>= 0		- the set baud rate (speed).
 * 	< 0		- follow the rerror code
 */
long sio_getbaud(int port);

/*
 * int sio_getmode(int port)
 *
 * To get the set data bits, parity bit & stop bit at now.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * Output:
 * 	>= 0		- the set data bits, parity bit & stop bit. The define is above.
 * 	< 0		- follow the rerror code
 */
int  sio_getmode(int port);

/*
 * int sio_get_flow(int port)
 *
 * To get the set flow control mode at now.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * Output:
 * 	>= 0		- the set flow control mode. The define is above.
 * 	< 0		- follow the rerror code
 */
int  sio_getflow(int port);

/*
 * int sio_DTR(int port, int mode)
 *
 * To control the modem signal DTR on or off.
 * Intput:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * 	int mode	- 0 for off, 1 for on.
 * Output:
 * 	Follow the error code.
 */
int  sio_DTR(int port, int mode);

/*
 * int sio_RTS(int port, int mode)
 *
 * To control the modem signal RTS on or off.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * 	int mode	- 0 for off, 1 for on.
 * 			  You can not control the RTS signal when the hardware flow control is enabled.
 * Output:
 * 	Follow the error code.
 */
int  sio_RTS(int port, int mode);

/*
 * int sio_baud(int port, long speed)
 *
 * To set the serial baud rate (speed).
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * 	long speed	- want to set baud rate (speed).
 * Output:
 * 	Follow the error code.
 */
int  sio_baud(int port, long speed);

/*
 * int sio_set_interface(int port, int mode)
 *
 * To set the serial device interface. You can set RS232, RS485-2-WIRE, RS422 or RS485-4-WIRE.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * 	int mode	- want to set interface. The define is above.
 * Output:
 * 	Follow the error code.
 */
int  sio_set_interface(int port, int mode);

/*
 * int sio_get_interface(int port, int *mode)
 *
 * To get the serial device interface at now.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * 	int *mode	- To save the set interface at now. The define is above.
 * Output:
 * 	Follow the error code.
 */
int  sio_get_interface(int port, int *mode);

/*
 * int sio_port_to_fd(int port)
 *
 * To get the file handle from serial port number.
 * Input:
 * 	int port	- the port number, from zero, up to the target's total port number.
 * Output:
 *	> 0		- the file handle
 *	< 0		- follow the error code.
 */
int  sio_port_fo_fd(int port);

/*
 * int sio_fd_to_fd(int fd)
 *
 * To get the serial port number from file handle.
 * Input:
 * 	int fd		- the file handle 
 * Output:
 *	>= 0		- the serial port number
 *	< 0		- follow the error code.
 */
int  sio_fd_to_port(int fd);

#ifdef __cplusplus
}
#endif
