
/*
 * This is a TCP client example.
 *
 * History:
 * Date		Author			Comment
 * 06-15-2006	Victor Yu.		Create it.
 */
#include	<stdio.h>
#include	<stdlib.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<unistd.h>
#include	<pthread.h>
#include	<time.h>
#include	<sys/types.h>
#include	<sys/socket.h>
#include	<sys/select.h>
#include	<linux/if.h>
#include	<netinet/in.h>
#include	<arpa/inet.h>
#include	<string.h>
#include	<pthread.h>
#include	"../serial/sio.h"

//#define DEBUG_ON		1
#ifdef DEBUG_ON
#define dbg_printf(x...)	printf(x)
#else
#define dbg_printf(x...)
#endif
#define BUF_LEN			4096
#define BUF_QUEUE(buf)		((buf)->wptr - (buf)->rptr)
#define BUF_FREE(buf)		(BUF_LEN - (buf)->wptr)
struct tcp_server_info;
struct buffer_struct {
	char			buffer[BUF_LEN];
	int			rptr, wptr;
};
struct tcp_server_info {
	// TCP information
	int			tcpport;	// TCP port number
	int			tcpfd;		// TCP client file handle
	// serial port information
	int			serport;	// serial port number
	int			serfd;		// serial port file handle
	int			speed;		// serial port baud rate
	int			serinterface;	// serial port interface
	int			flowcontrol;	// serial port flow control mode
	// buffer 
	struct buffer_struct	*txbuf, *rxbuf;
	// remote server IP address
	char			*ipaddr;
	// pthread file handle
	pthread_t		pthd;
};

static struct tcp_server_info	tcp_info[] = {
	{4001, 0,
	 0, 0, 115200, RS232_MODE, HW_FLOWCTRL,
	 NULL, NULL,
	 "192.168.27.130"},

	{4002, 0,
	 1, 0, 115200, RS232_MODE, HW_FLOWCTRL,
	 NULL, NULL,
	 "192.168.27.130"},

	{4003, 0,
	 2, 0, 115200, RS232_MODE, HW_FLOWCTRL,
	 NULL, NULL,
	 "192.168.27.130"},

	{4004, 0,
	 3, 0, 115200, RS232_MODE, HW_FLOWCTRL,
	 NULL, NULL,
	 "192.168.27.130"},
};
#define TCP_NO	(sizeof(tcp_info)/sizeof(struct tcp_server_info))

static void	check_and_clear_buffer(struct buffer_struct *buf)
{
	if ( buf->rptr == BUF_LEN && buf->wptr == BUF_LEN )
		buf->rptr = buf->wptr = 0;
}

static void	data_process(struct tcp_server_info *tcp)
{
	fd_set		readfds, writefds;
	struct timeval	time;
	int		j, maxfd;

	while ( 1 ) {
		// initialize the file handle to select
		time.tv_usec = 0;
		time.tv_sec = 1;
		maxfd = 0;
		FD_ZERO(&readfds);
		FD_ZERO(&writefds);

		if ( tcp->tcpfd > maxfd )
			maxfd = tcp->tcpfd;
		if ( BUF_QUEUE(tcp->txbuf) )
			FD_SET(tcp->tcpfd, &writefds);
		if ( BUF_FREE(tcp->rxbuf) )
			FD_SET(tcp->tcpfd, &readfds);

		if ( tcp->serfd > maxfd )
			maxfd = tcp->serfd;
		if ( BUF_FREE(tcp->txbuf) )
			FD_SET(tcp->serfd, &readfds);
		if ( BUF_QUEUE(tcp->rxbuf) )
			FD_SET(tcp->serfd, &writefds);

		// wait select return
		if ( select(maxfd+1, &readfds, &writefds, NULL, &time) >= 0 ) {
			if ( FD_ISSET(tcp->serfd, &readfds) ) {
				j = read(tcp->serfd, &tcp->txbuf->buffer[tcp->txbuf->wptr], BUF_FREE(tcp->txbuf));
				tcp->txbuf->wptr += j;
			}
			if ( FD_ISSET(tcp->serfd, &writefds) ) {
				j = write(tcp->serfd, &tcp->rxbuf->buffer[tcp->rxbuf->rptr], BUF_QUEUE(tcp->rxbuf));
				tcp->rxbuf->rptr += j;
				check_and_clear_buffer(tcp->rxbuf);
			}

			if ( FD_ISSET(tcp->tcpfd, &readfds) ) {
				j = read(tcp->tcpfd, &tcp->rxbuf->buffer[tcp->rxbuf->wptr], BUF_FREE(tcp->rxbuf));
				if ( j <= 0 ) {
					dbg_printf("TCP port [%d] disconnect !\n", tcp->tcpport);
					return;
				}
				tcp->rxbuf->wptr += j;
			}
			if ( FD_ISSET(tcp->tcpfd, &writefds) ) {
				j = write(tcp->tcpfd, &tcp->txbuf->buffer[tcp->txbuf->rptr], BUF_QUEUE(tcp->txbuf));
				if ( j <= 0 ) {
					dbg_printf("TCP port [%d] disconnect !\n", tcp->tcpport);
					return;
				}
				tcp->txbuf->rptr += j;
				check_and_clear_buffer(tcp->txbuf);
			}
		} // end if select
	}
}

static void	*tcp_thread(void *ptr)
{
	int			j;
	struct sockaddr_in	des;
	struct tcp_server_info	*tcp=(struct tcp_server_info *)ptr;

	while ( 1 ) {
		tcp->tcpfd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
		if ( tcp->tcpfd <= 0 ) {
			dbg_printf("Open socket fail !\n");
			sleep(1);
			continue;
		}
		des.sin_family = AF_INET;
		des.sin_port = htons(tcp->tcpport);
		des.sin_addr.s_addr = inet_addr(tcp->ipaddr);
		j = sizeof(des);
		while ( connect(tcp->tcpfd, (struct sockaddr *)&des, j) < 0 ) {
			dbg_printf("TCP port [%d] connect to server fail !\n", tcp->tcpport);
			sleep(1);
		}
		dbg_printf("TCP port [%d] connect OK.\n", tcp->tcpport);

		// RS232 initialize
		if ( sio_open(tcp->serport) != SIO_OK ) {
			dbg_printf("Open serial port [%d] fail !\n", tcp->serport);
			close(tcp->tcpfd);
			sleep(1);
			continue;
		}
		sio_baud(tcp->serport, tcp->speed);
		sio_set_interface(tcp->serport, tcp->serinterface);
		sio_flowctrl(tcp->serport, tcp->flowcontrol);
		sio_flush(tcp->serport, FLUSH_INOUT);
		tcp->serfd = sio_port_to_fd(tcp->serport);

		// start to receive/transfer data
		data_process(tcp);
		dbg_printf("TCP port [%d] data process return !\n", tcp->tcpport);

		// stop data processing
		tcp->txbuf->rptr = tcp->txbuf->wptr = 0;
		tcp->rxbuf->rptr = tcp->rxbuf->wptr = 0;
		dbg_printf("TCP port [%d], serial port [%d] flushing serial !\n", tcp->tcpport, tcp->serport);
		sio_flush(tcp->serport, FLUSH_INOUT);
		dbg_printf("TCP port [%d], serial port [%d] closeing serial !\n", tcp->tcpport, tcp->serport);
		sio_close(tcp->serport);
		close(tcp->tcpfd);
		dbg_printf("TCP port [%d] restart !\n", tcp->tcpport);
	}
	return NULL;
}

static int	tcp_init(void)
{
	int			i;
	struct tcp_server_info	*tcp;

	for ( i=0; i<TCP_NO; i++ ) {
		tcp = &tcp_info[i];
		tcp->txbuf = malloc(sizeof(struct buffer_struct));
		if ( tcp->txbuf == NULL ) {
			printf("Allocate transmit buffer fail !\n");
			return -1;
		}
		memset(tcp->txbuf, 0, sizeof(struct buffer_struct));
		tcp->rxbuf = malloc(sizeof(struct buffer_struct));
		if ( tcp->rxbuf == NULL ) {
			printf("Allocate transmit buffer fail !\n");
			return -1;
		}
		memset(tcp->rxbuf, 0, sizeof(struct buffer_struct));
		if ( pthread_create(&tcp->pthd, NULL, tcp_thread, (void *)tcp) < 0 ) {
			printf("Pthread create fail !\n");
			return -1;
		}
		dbg_printf("TCP port [%d] initialize OK\n", tcp->tcpport);
	}
}

int	main(int argc, char *argv[])
{
	int			i;
	void			*ret;
	struct tcp_server_info	*tcp;

	dbg_printf("Startig the program !\n");
	if ( tcp_init() < 0 )
		return -1;
	dbg_printf("TCP initialize OK [%d].\n", TCP_NO);

	for ( i=0; i<TCP_NO; i++ ) {
		tcp = &tcp_info[i];
		pthread_join(tcp->pthd, &ret);
	} // end if select
	sio_release();
	return 0;
}
