
/*
 * This is a TCP server example.
 *
 * History:
 * Date		Author			Comment
 * 06-15-2006	Victor Yu.		Create it.
 */
#include	<stdio.h>
#include	<stdlib.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<unistd.h>
#include	<pthread.h>
#include	<time.h>
#include	<sys/types.h>
#include	<sys/socket.h>
#include	<sys/select.h>
#include	<linux/if.h>
#include	<netinet/in.h>
#include	<arpa/inet.h>
#include	<string.h>
#include	"../serial/sio.h"

//#define DEBUG_ON		1
#ifdef DEBUG_ON
#define dbg_printf(x...)	printf(x)
#else
#define dbg_printf(x...)
#endif
#define BUF_LEN			4096
#define BUF_QUEUE(buf)		((buf)->wptr - (buf)->rptr)
#define BUF_FREE(buf)		(BUF_LEN - (buf)->wptr)
struct tcp_server_info;
struct buffer_struct {
	char			buffer[BUF_LEN];
	int			rptr, wptr;
};
struct accept_info {
	int			fd;		// TCP accept file handle
	struct buffer_struct	*rxbuf;
	struct tcp_server_info	*tcp;
	struct accept_info	*next;
};
struct tcp_server_info {
	// TCP information
	int			tcpport;	// TCP port number
	int			tcpfd;		// TCP server file handle
	// serial port information
	int			serport;	// serial port number
	int			serfd;		// serial port file handle
	int			speed;		// serial port baud rate
	int			serinterface;	// serial port interface
	int			flowcontrol;	// serial port flow control mode
	// buffer 
	struct buffer_struct	*txbuf;
	// TCP accept information
	struct accept_info	*acphead, *acptail;
};

static struct tcp_server_info	tcp_info[] = {
	{4001, 0,
	 0, 0, 115200, RS232_MODE, HW_FLOWCTRL,
	 NULL,
	 NULL, NULL},

	{4002, 0,
	 1, 0, 115200, RS232_MODE, HW_FLOWCTRL,
	 NULL,
	 NULL, NULL},

	{4003, 0,
	 2, 0, 115200, RS232_MODE, HW_FLOWCTRL,
	 NULL,
	 NULL, NULL},

	{4004, 0,
	 3, 0, 115200, RS232_MODE, HW_FLOWCTRL,
	 NULL,
	 NULL, NULL},
};
#define TCP_NO	(sizeof(tcp_info)/sizeof(struct tcp_server_info))

static void	check_and_clear_buffer(struct buffer_struct *buf)
{
	if ( buf->rptr == BUF_LEN && buf->wptr == BUF_LEN )
		buf->rptr = buf->wptr = 0;
}

static int	tcp_init(void)
{
	int			i, j;
	struct tcp_server_info	*tcp;
	struct sockaddr_in	so;

	for ( i=0; i<TCP_NO; i++ ) {
		tcp = &tcp_info[i];
		tcp->txbuf = malloc(sizeof(struct buffer_struct));
		if ( tcp->txbuf == NULL ) {
			printf("Allocate transmit buffer fail !\n");
			return -1;
		}
		memset(tcp->txbuf, 0, sizeof(struct buffer_struct));
		tcp->tcpfd = socket(AF_INET, SOCK_STREAM, IPPROTO_TCP);
		if ( tcp->tcpfd <= 0 ) {
			printf("Open socket fail !\n");
			return -1;
		}
		memset((char *)&so, 0, sizeof(so));
		so.sin_family = AF_INET;
		so.sin_port = htons(tcp->tcpport);
		so.sin_addr.s_addr = INADDR_ANY;
		j = sizeof(so);
		if ( bind(tcp->tcpfd, (struct sockaddr *)&so, j) < 0 ) {
			printf("Bind socket fail !\n");
			return -1;
		}
		if ( listen(tcp->tcpfd, 1) < 0 ) {
			printf("Listen fail !\n");
			return -1;
		}
		dbg_printf("TCP port [%d] initialize OK, fd = %d\n", tcp->tcpport, tcp->tcpfd);
	}
}

static void	del_accept(struct tcp_server_info *tcp, int fd)
{
	struct accept_info	*now, *priv;

	dbg_printf("Del one connect [%d]\n", tcp->tcpport);
	now = priv = tcp->acphead;
	while ( now ) {
		if ( now->fd == fd ) {
                        if ( now == tcp->acphead ) {
                                if ( tcp->acptail == tcp->acphead )
                                        tcp->acptail = NULL;
                                tcp->acphead = tcp->acphead->next;
                        } else if ( now == tcp->acptail )
                                tcp->acptail = priv;
                        priv->next = now->next;
                        close(now->fd);
			free(now->rxbuf);
                        free(now);
                        break;
                }
                priv = now;
                now = now->next;
	}
	if ( tcp->acphead == NULL ) {
		dbg_printf("TCP port not any connecteion [%d] !\n", tcp->tcpport);
		tcp->txbuf->rptr = tcp->txbuf->wptr = 0;
		sio_flush(tcp->serport, FLUSH_INOUT);
		sio_close(tcp->serport);
		tcp->serfd = 0;
	}
}

static int	add_accept(struct tcp_server_info *tcp, int fd)
{
	struct accept_info	*acp;

	dbg_printf("Add one connection [tcp=%d]\n", tcp->tcpport);
	acp = malloc(sizeof(struct accept_info));
	if ( acp == NULL ) {
		dbg_printf("Allocate memory for new accept fail !\n");
		return -1;
	}
	memset((char *)acp, 0, sizeof(struct accept_info));
	acp->rxbuf = malloc(sizeof(struct buffer_struct));
	if ( acp->rxbuf == NULL ) {
		printf("Allocate receive buffer fail !\n");
		free(acp);
		return -1;
	}
	memset(acp->rxbuf, 0, sizeof(struct buffer_struct));
	acp->tcp = tcp;
	acp->fd = fd;
	if ( tcp->acphead == NULL ) {
		dbg_printf("TCP [%d] add accept head !\n", tcp->tcpport);
		if ( sio_open(tcp->serport) != SIO_OK ) {
			dbg_printf("Open serial port [%d] fail !\n", tcp->serport);
			free(acp->rxbuf);
			free(acp);
			return -1;
		}
		sio_baud(tcp->serport, tcp->speed);
		sio_set_interface(tcp->serport, tcp->serinterface);
		sio_flowctrl(tcp->serport, tcp->flowcontrol);
		sio_flush(tcp->serport, FLUSH_INOUT);
		tcp->serfd = sio_port_to_fd(tcp->serport);
		tcp->acphead = acp;
	} else {
		dbg_printf("TCP [%d] add accept tail !\n", tcp->tcpport);
		tcp->acptail->next = acp;
	}
	tcp->acptail = acp;

	dbg_printf("Add new connection OK. [tcp=%d, fd=%d]\n", tcp->tcpport, acp->fd);
	return 0;
}

int	main(int argc, char *argv[])
{
	fd_set			readfds, writefds;
	struct timeval		time;
	int			i, j, maxfd, maxlen;
	struct tcp_server_info	*tcp;
	struct accept_info	*acp, *temp;
	struct sockaddr		sa;

	dbg_printf("Startig the program !\n");
	if ( tcp_init() < 0 )
		return -1;
	dbg_printf("TCP initialize OK [%d].\n", TCP_NO);

	while ( 1 ) {
		// initialize the file handle to select
		time.tv_usec = 0;
		time.tv_sec = 1;
		maxfd = 0;
		FD_ZERO(&readfds);
		FD_ZERO(&writefds);
		for ( i=0; i<TCP_NO; i++ ) {
			tcp = &tcp_info[i];
			FD_SET(tcp->tcpfd, &readfds);
			if ( tcp->tcpfd > maxfd )
				maxfd = tcp->tcpfd;
			acp = tcp->acphead;
			if ( acp ) {
				if ( tcp->serfd > maxfd )
					maxfd = tcp->serfd;
				if ( BUF_FREE(tcp->txbuf) )
					FD_SET(tcp->serfd, &readfds);
			}
			while ( acp ) {
				if ( acp->fd > maxfd )
					maxfd = acp->fd;
				if ( BUF_FREE(acp->rxbuf) )
					FD_SET(acp->fd, &readfds);
				if ( BUF_QUEUE(acp->rxbuf) )
					FD_SET(tcp->serfd, &writefds);
				if ( BUF_QUEUE(tcp->txbuf) )
					FD_SET(acp->fd, &writefds);
				acp = acp->next;
			}
		} // end select for loop

		// wait select return
		if ( select(maxfd+1, &readfds, &writefds, NULL, &time) >= 0 ) {
			for ( i=0; i<TCP_NO; i++ ) {
				tcp = &tcp_info[i];
				acp = tcp->acphead;
				if ( acp ) {
					if ( FD_ISSET(tcp->serfd, &readfds) ) {
						j = read(tcp->serfd, &tcp->txbuf->buffer[tcp->txbuf->wptr], BUF_FREE(tcp->txbuf));
						tcp->txbuf->wptr += j;
					}
					if ( FD_ISSET(tcp->serfd, &writefds) ) {
						while ( acp ) {
							j = write(tcp->serfd, &acp->rxbuf->buffer[acp->rxbuf->rptr], BUF_QUEUE(acp->rxbuf));
							acp->rxbuf->rptr += j;
							check_and_clear_buffer(acp->rxbuf);
							acp = acp->next;
						}
					}
				} // end if acp

				if ( FD_ISSET(tcp->tcpfd, &readfds) ) {
					dbg_printf("TCP port [%d] new connection coming !\n", tcp->tcpport);
					j = accept(tcp->tcpfd, &sa, &maxlen);
					if ( add_accept(tcp, j) < 0 ) {
						dbg_printf("Add one connect fail [%d] !\n", tcp->tcpport);
						close(j);
					}
				}

				maxlen = 0;
				acp = tcp->acphead;
				while ( acp ) {
					if ( FD_ISSET(acp->fd, &readfds) ) {
						j = read(acp->fd, &acp->rxbuf->buffer[acp->rxbuf->wptr], BUF_FREE(acp->rxbuf));
						if ( j <= 0 ) {
acp_disconnect:
							// disconnect
							temp = acp;
							acp = acp->next;
							dbg_printf("TCP port %d disconnect !\n", tcp->tcpport);
							del_accept(tcp, temp->fd);
							continue;
						}
						acp->rxbuf->wptr += j;
					}
					if ( FD_ISSET(acp->fd, &writefds) ) {
						j = write(acp->fd, &tcp->txbuf->buffer[tcp->txbuf->rptr], BUF_QUEUE(tcp->txbuf));
						if ( j <= 0 )
							goto acp_disconnect;
						if ( j > maxlen )
							maxlen = j;
					}
					acp = acp->next;
				} // end while acp
				if ( tcp->acphead ) {
					tcp->txbuf->rptr += maxlen;
					check_and_clear_buffer(tcp->txbuf);
				}
			} // end for loop
		} // end if select
	}
	sio_release();
	return 0;
}
