
/*
 * This is a UDP client example.
 *
 * History:
 * Date		Author			Comment
 * 06-19-2006	Victor Yu.		Create it.
 */
#include	<stdio.h>
#include	<stdlib.h>
#include	<fcntl.h>
#include	<errno.h>
#include	<unistd.h>
#include	<pthread.h>
#include	<time.h>
#include	<sys/types.h>
#include	<sys/socket.h>
#include	<sys/select.h>
#include	<linux/if.h>
#include	<netinet/in.h>
#include	<arpa/inet.h>
#include	<string.h>
#include	<pthread.h>
#include	"../serial/sio.h"

//#define DEBUG_ON		1
#ifdef DEBUG_ON
#define dbg_printf(x...)	printf(x)
#else
#define dbg_printf(x...)
#endif
#define BUF_LEN			4096
#define BUF_QUEUE(buf)		((buf)->wptr - (buf)->rptr)
#define BUF_FREE(buf)		(BUF_LEN - (buf)->wptr)
struct tcp_server_info;
struct buffer_struct {
	char			buffer[BUF_LEN];
	int			rptr, wptr;
};
struct udp_info_struct {
	// UDP information
	int			udpport;	// UDP port number
	int			udpfd;		// UDP client file handle
	// serial port information
	int			serport;	// serial port number
	int			serfd;		// serial port file handle
	int			speed;		// serial port baud rate
	int			serinterface;	// serial port interface
	int			flowcontrol;	// serial port flow control mode
	// buffer 
	struct buffer_struct	*txbuf, *rxbuf;
	// remote server IP address
	char			*ipaddr;
	// pthread file handle
	pthread_t		pthd;
};

static struct udp_info_struct	udp_info[] = {
	{4001, 0,
	 0, 0, 115200, RS232_MODE, HW_FLOWCTRL,
	 NULL, NULL,
	 "192.168.27.130"},

	{4002, 0,
	 1, 0, 115200, RS232_MODE, HW_FLOWCTRL,
	 NULL, NULL,
	 "192.168.27.130"},

	{4003, 0,
	 2, 0, 115200, RS232_MODE, HW_FLOWCTRL,
	 NULL, NULL,
	 "192.168.27.130"},

	{4004, 0,
	 3, 0, 115200, RS232_MODE, HW_FLOWCTRL,
	 NULL, NULL,
	 "192.168.27.130"},
};
#define UDP_NO	(sizeof(udp_info)/sizeof(struct udp_info_struct))

static void	check_and_clear_buffer(struct buffer_struct *buf)
{
	if ( buf->rptr == BUF_LEN && buf->wptr == BUF_LEN )
		buf->rptr = buf->wptr = 0;
}

static void	data_process(struct udp_info_struct *udp)
{
	fd_set			readfds, writefds;
	struct timeval		time;
	int			j, maxfd;
	struct sockaddr_in	des, from;
	int			deslen, fromlen;

	des.sin_family = AF_INET;
	des.sin_port = htons(udp->udpport);
	des.sin_addr.s_addr = inet_addr(udp->ipaddr);
	deslen = sizeof(des);
	while ( 1 ) {
		// initialize the file handle to select
		time.tv_usec = 0;
		time.tv_sec = 1;
		maxfd = 0;
		FD_ZERO(&readfds);
		FD_ZERO(&writefds);

		if ( udp->udpfd > maxfd )
			maxfd = udp->udpfd;
		if ( BUF_QUEUE(udp->txbuf) )
			FD_SET(udp->udpfd, &writefds);
		if ( BUF_FREE(udp->rxbuf) )
			FD_SET(udp->udpfd, &readfds);

		if ( udp->serfd > maxfd )
			maxfd = udp->serfd;
		if ( BUF_FREE(udp->txbuf) )
			FD_SET(udp->serfd, &readfds);
		if ( BUF_QUEUE(udp->rxbuf) )
			FD_SET(udp->serfd, &writefds);

		// wait select return
		if ( select(maxfd+1, &readfds, &writefds, NULL, &time) >= 0 ) {
			// serial port read/write data
			if ( FD_ISSET(udp->serfd, &readfds) ) {
				j = read(udp->serfd, &udp->txbuf->buffer[udp->txbuf->wptr], BUF_FREE(udp->txbuf));
				udp->txbuf->wptr += j;
			}
			if ( FD_ISSET(udp->serfd, &writefds) ) {
				j = write(udp->serfd, &udp->rxbuf->buffer[udp->rxbuf->rptr], BUF_QUEUE(udp->rxbuf));
				udp->rxbuf->rptr += j;
				check_and_clear_buffer(udp->rxbuf);
			}

			// networking read/write data
			if ( FD_ISSET(udp->udpfd, &readfds) ) {
				j = recvfrom(udp->udpfd, &udp->rxbuf->buffer[udp->rxbuf->wptr], BUF_FREE(udp->rxbuf), 0, (struct sockaddr *)&from, &fromlen);
				udp->rxbuf->wptr += j;
			}
			if ( FD_ISSET(udp->udpfd, &writefds) ) {
				j = sendto(udp->udpfd, &udp->txbuf->buffer[udp->txbuf->rptr], BUF_QUEUE(udp->txbuf), 0, (struct sockaddr *)&des, deslen);
				udp->txbuf->rptr += j;
				check_and_clear_buffer(udp->txbuf);
			}
		} // end if select
	}
}

static void	*udp_thread(void *ptr)
{
	int			i;
	struct udp_info_struct	*udp=(struct udp_info_struct *)ptr;
	struct sockaddr_in	so;

	while ( 1 ) {
		udp->udpfd = socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP);
		if ( udp->udpfd <= 0 ) {
			dbg_printf("Open socket fail !\n");
			sleep(1);
			continue;
		}
		memset((char *)&so, 0, sizeof(so));
		so.sin_family = AF_INET;
		so.sin_port = htons(udp->udpport);
		so.sin_addr.s_addr = INADDR_ANY;
		i = sizeof(so);
		if ( bind(udp->udpfd, (struct sockaddr *)&so, i) < 0 ) {
			printf("Bind socket fail !\n");
			close(udp->udpfd);
			sleep(1);
			continue;
		}

		// RS232 initialize
		if ( sio_open(udp->serport) != SIO_OK ) {
			dbg_printf("Open serial port [%d] fail !\n", udp->serport);
			close(udp->udpfd);
			sleep(1);
			continue;
		}
		sio_baud(udp->serport, udp->speed);
		sio_set_interface(udp->serport, udp->serinterface);
		sio_flowctrl(udp->serport, udp->flowcontrol);
		sio_flush(udp->serport, FLUSH_INOUT);
		udp->serfd = sio_port_to_fd(udp->serport);

		// start to receive/transfer data
		data_process(udp);
		dbg_printf("UDP port [%d] data process return !\n", udp->udpport);

		// stop data processing
		udp->txbuf->rptr = udp->txbuf->wptr = 0;
		udp->rxbuf->rptr = udp->rxbuf->wptr = 0;
		dbg_printf("UDP port [%d], serial port [%d] flushing serial !\n", udp->udpport, udp->serport);
		sio_flush(udp->serport, FLUSH_INOUT);
		dbg_printf("UDP port [%d], serial port [%d] closeing serial !\n", udp->udpport, udp->serport);
		sio_close(udp->serport);
		close(udp->udpfd);
		dbg_printf("UDP port [%d] restart !\n", udp->udpport);
	}
	return NULL;
}

static int	udp_init(void)
{
	int			i, j;
	struct udp_info_struct	*udp;
	struct sockaddr_in	des;

	for ( i=0; i<UDP_NO; i++ ) {
		udp = &udp_info[i];
		udp->txbuf = malloc(sizeof(struct buffer_struct));
		if ( udp->txbuf == NULL ) {
			printf("Allocate transmit buffer fail !\n");
			return -1;
		}
		memset(udp->txbuf, 0, sizeof(struct buffer_struct));
		udp->rxbuf = malloc(sizeof(struct buffer_struct));
		if ( udp->rxbuf == NULL ) {
			printf("Allocate transmit buffer fail !\n");
			return -1;
		}
		memset(udp->rxbuf, 0, sizeof(struct buffer_struct));
		if ( pthread_create(&udp->pthd, NULL, udp_thread, (void *)udp) < 0 ) {
			printf("Pthread create fail !\n");
			return -1;
		}
		dbg_printf("UDP port [%d] initialize OK\n", udp->udpport);
	}
}

int	main(int argc, char *argv[])
{
	int			i;
	void			*ret;
	struct udp_info_struct	*udp;

	dbg_printf("Startig the program !\n");
	if ( udp_init() < 0 )
		return -1;
	dbg_printf("UDP initialize OK [%d].\n", UDP_NO);

	for ( i=0; i<UDP_NO; i++ ) {
		udp = &udp_info[i];
		pthread_join(udp->pthd, &ret);
	} // end if select
	sio_release();
	return 0;
}
